import csv
import datetime
import ladok3
import os
import sys

def extract_data_for_round(ladok, course_round):
  course_start = course_round.start
  course_length = course_round.end - course_start
  component = course_round.components()[0]
  results = ladok.search_reported_results_JSON(
    course_round.round_id, component.instance_id)

  for result in results:
    student = result["Student"]["Uid"]

    for component_result in result["ResultatPaUtbildningar"]:
      if component_result["HarTillgodoraknande"]:
        continue

      if "Arbetsunderlag" in component_result:
        result_data = component_result["Arbetsunderlag"]
      elif "SenastAttesteradeResultat" in component_result:
        result_data = component_result["SenastAttesteradeResultat"]
      else:
        continue
      matching_component = course_round.components(
        instance_id=result_data["UtbildningsinstansUID"])
      if len(matching_component) < 1:
        continue
      component_code = matching_component[0].code
      if "Betygsgradsobjekt" in result_data:
        grade = result_data["Betygsgradsobjekt"]["Kod"]
        date = datetime.date.fromisoformat(
          result_data["Examinationsdatum"])
        normalized_date = (date - course_start) / course_length
      else:
        grade = "-"
        normalized_date = None

      yield (student, component_code, grade, normalized_date)
def clean_data(data):
  data = list(data)
  students_to_remove = reregistered_students(data)
  return remove_students(students_to_remove, data)
def reregistered_students(data):
  students = set()
  for student, _, _, time in data:
    if time and time < 0:
      students.add(student)
  return students

def remove_students(students, data):
  for row in data:
    if row[0] not in students:
      yield row

def add_command_options(parser):
  data_parser = parser.add_parser("data",
    help="Returns course results data in CSV form")
  data_parser.set_defaults(func=command)
  data_parser.add_argument("course_code",
    help="The course code of the course for which to export data")

def command(ladok, args):
  data_writer = csv.writer(sys.stdout)
  course_rounds = ladok.search_course_rounds(code=args.course_code)

  data_writer.writerow([
    "Course", "Round", "Component", "Student", "Grade", "Time"
  ])
  for course_round in course_rounds:
    data = extract_data_for_round(ladok, course_round)
    data = clean_data(data)

    for student, component, grade, time in data:
      data_writer.writerow(
        [course_round.code, course_round.round_code, component,
          student, grade, time]
      )



#!/bin/env python3
"""A command-line interface for LADOK 3"""

import appdirs
import argcomplete, argparse
import json
import os
import pickle
import re
import sys
import traceback

import ladok3.data
import ladok3.report

dirs = appdirs.AppDirs("ladok", "dbosk@kth.se")

def err(rc, msg):
  print(f"{sys.argv[0]}: error: {msg}", file=sys.stderr)
  traceback.print_exc(file=sys.stderr)
  sys.exit(rc)

def warn(msg):
  print(f"{sys.argv[0]}: {msg}", file=sys.stderr)
def store_ladok_session(ls):
  if not os.path.isdir(dirs.user_cache_dir):
    os.mkdir(dirs.user_cache_dir)

  file_path = dirs.user_cache_dir + "/LadokSession"

  with open(file_path, "wb") as file:
    pickle.dump(ls, file)
  
def restore_ladok_session():
  file_path = dirs.user_cache_dir + "/LadokSession"

  if os.path.isfile(file_path):
    with open(file_path, "rb") as file:
      return pickle.load(file)

  return None
def load_credentials(filename):
  """Load credentials from environment or file named filename"""
  try:
    user = os.environ["LADOK_USER"]
    passwd = os.environ["LADOK_PASS"]
    return user, passwd
  except:
    pass

  try:
    with open(filename) as conf_file:
      config = json.load(conf_file)
    return config["username"], config["password"]
  except:
    raise Exception("Cannot find credentials.")
def clear_cache(ls, args):
  os.remove(dirs.user_cache_dir + "/LadokSession")
  sys.exit(0)
def main():
  """Run the command-line interface for the ladok command"""
  argp = argparse.ArgumentParser(
    description="Command-line interface for LADOK3",
    epilog="Web: https://github.com/dbosk/ladok3"
  )
  argp.add_argument("-f", "--config-file",
    default=f"{dirs.user_config_dir}/config.json",
    help="Path to configuration file "
      f"(default: {dirs.user_config_dir}/config.json) "
      "or set LADOK_USER and LADOK_PASS environment variables.")
  subp = argp.add_subparsers(
    title="commands",
    dest="command",
    required=True
  )
  cache_parser = subp.add_parser("cache",
    help="Manage cache",
    description="Manages the cache of LADOK data"
  )
  cache_subp = cache_parser.add_subparsers(
    title="subcommands",
    dest="subcommand",
    required=True
  )
  cache_clear = cache_subp.add_parser("clear",
    help="Clear the cache",
    description="Clears everything from the cache"
  )
  cache_clear.set_defaults(func=clear_cache)
  ladok3.data.add_command_options(subp)
  ladok3.report.add_command_options(subp)
  argcomplete.autocomplete(argp)
  args = argp.parse_args()
  LADOK_USER, LADOK_PASS = load_credentials(args.config_file)
  ls = restore_ladok_session()
  if not ls:
    ls = ladok3.LadokSessionKTH(LADOK_USER, LADOK_PASS)
  if "func" in args:
    args.func(ls, args)
  store_ladok_session(ls)

if __name__ == "__main__":
  try:
    main()
    sys.exit(0)
  except Exception as e:
    err(-1, e)



import datetime
import ladok3

def add_command_options(parser):
  report_parser = parser.add_parser("report",
    help="Reports course results to LADOK",
    description="Reports course results to LADOK"
  )
  report_parser.set_defaults(func=command)
  report_parser.add_argument("student_id",
    help="Student identifier (personnummer or LADOK ID)"
  )

  report_parser.add_argument("course_code",
    help="The course code (e.g. DD1315) for which the grade is for"
  )

  report_parser.add_argument("component_code",
    help="The component code (e.g. LAB1) for which the grade is for"
  )

  report_parser.add_argument("grade",
    help="The grade (e.g. A or P)"
  )
  report_parser.add_argument("-d", "--date",
    help="Date on ISO format (e.g. 2021-03-18), "
      f"defaults to today's date ({datetime.date.today()})",
    type=datetime.date.fromisoformat,
    default=datetime.date.today()
  )
  report_parser.add_argument("-f", "--finalize",
    help="Finalize the grade (mark as ready/klarmarkera) for examiner to attest",
    action="store_true",
    default=False
  )

def command(ladok, args):
  try:
    student = ladok.get_student(args.student_id)
    course = student.courses(code=args.course_code)[0]
    result = course.results(component=args.component_code)[0]
    result.set_grade(args.grade, args.date)
    if args.finalize:
      result.finalize()
  except Exception as err:
    try:
      print(f"{student}: {err}")
    except ValueError as verr:
      print(f"{verr}: {args.student_id}: {err}")
